@@ -29,6 +29,7 @@
 #include <QPainter>
 #include <QFileDialog>
 #include <QBuffer>
+#include <QRegularExpression>
 
 #include <KLocalizedString>
 #include <KSharedConfig>
@@ -41,9 +42,14 @@
 ExportManager::ExportManager(QObject *parent) :
     QObject(parent),
     mSavePixmap(QPixmap()),
+    mLastSavePath(QUrl()),
     mTempFile(QUrl()),
     mTempDir(nullptr)
-{}
+{
+    connect(this, &ExportManager::imageSaved, [this](const QUrl savedAt) {
+        mLastSavePath = savedAt;
+    });
+}
 
 ExportManager::~ExportManager()
 {
@@ -82,6 +88,26 @@ QString ExportManager::pixmapDataUri() const
     return uri;
 }
 
+void ExportManager::setWindowTitle(const QString &windowTitle)
+{
+    mWindowTitle = windowTitle;
+}
+
+QString ExportManager::windowTitle() const
+{
+    return mWindowTitle;
+}
+
+ImageGrabber::GrabMode ExportManager::grabMode() const
+{
+    return mGrabMode;
+}
+
+void ExportManager::setGrabMode(const ImageGrabber::GrabMode &grabMode)
+{
+    mGrabMode = grabMode;
+}
+
 void ExportManager::setPixmap(const QPixmap &pixmap)
 {
     mSavePixmap = pixmap;
@@ -118,6 +144,11 @@ QString ExportManager::saveLocation() const
     return savePath;
 }
 
+QUrl ExportManager::lastSavePath() const
+{
+    return isFileExists(mLastSavePath) ? mLastSavePath : QUrl();
+}
+
 void ExportManager::setSaveLocation(const QString &savePath)
 {
     KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("spectaclerc"));
@@ -143,6 +174,17 @@ QUrl ExportManager::getAutosaveFilename()
     }
 }
 
+QString ExportManager::truncatedFilename(QString const &filename)
+{
+    QString result = filename;
+    constexpr auto maxFilenameLength = 255;
+    constexpr auto maxExtensionLength = 5;  // For example, ".jpeg"
+    constexpr auto maxCounterLength = 20;  // std::numeric_limits<quint64>::max() == 18446744073709551615
+    constexpr auto maxLength = maxFilenameLength - maxCounterLength - maxExtensionLength;
+    result.truncate(maxLength);
+    return result;
+}
+
 QString ExportManager::makeAutosaveFilename()
 {
     KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("spectaclerc"));
@@ -151,33 +193,56 @@ QString ExportManager::makeAutosaveFilename()
     const QDateTime timestamp = QDateTime::currentDateTime();
     QString baseName = generalConfig.readEntry("save-filename-format", "Screenshot_%Y%M%D_%H%m%S");
 
-    return baseName.replace(QLatin1String("%Y"), timestamp.toString(QStringLiteral("yyyy")))
-                   .replace(QLatin1String("%y"), timestamp.toString(QStringLiteral("yy")))
-                   .replace(QLatin1String("%M"), timestamp.toString(QStringLiteral("MM")))
-                   .replace(QLatin1String("%D"), timestamp.toString(QStringLiteral("dd")))
-                   .replace(QLatin1String("%H"), timestamp.toString(QStringLiteral("hh")))
-                   .replace(QLatin1String("%m"), timestamp.toString(QStringLiteral("mm")))
-                   .replace(QLatin1String("%S"), timestamp.toString(QStringLiteral("ss")));
+    QString title;
+
+    if (mGrabMode == ImageGrabber::GrabMode::ActiveWindow ||
+        mGrabMode == ImageGrabber::GrabMode::TransientWithParent ||
+        mGrabMode == ImageGrabber::GrabMode::WindowUnderCursor) {
+        title = mWindowTitle;
+    } else {
+        // Remove '%T' with separators around it
+        const auto wordSymbol = QStringLiteral(R"(\p{L}\p{M}\p{N})");
+        const auto separator = QStringLiteral("([^%1]+)").arg(wordSymbol);
+        const auto re = QRegularExpression(QStringLiteral("(.*?)(%1%T|%T%1)(.*?)").arg(separator));
+        baseName.replace(re, QStringLiteral(R"(\1\5)"));
+    }
+
+    QString result = baseName.replace(QLatin1String("%Y"), timestamp.toString(QStringLiteral("yyyy")))
+                             .replace(QLatin1String("%y"), timestamp.toString(QStringLiteral("yy")))
+                             .replace(QLatin1String("%M"), timestamp.toString(QStringLiteral("MM")))
+                             .replace(QLatin1String("%D"), timestamp.toString(QStringLiteral("dd")))
+                             .replace(QLatin1String("%H"), timestamp.toString(QStringLiteral("hh")))
+                             .replace(QLatin1String("%m"), timestamp.toString(QStringLiteral("mm")))
+                             .replace(QLatin1String("%S"), timestamp.toString(QStringLiteral("ss")))
+                             .replace(QLatin1String("%T"), title)
+                             .replace(QLatin1String("/"), QLatin1String("_"));  // POSIX doesn't allow "/" in filenames
+    if (result.isEmpty()) {
+        result = QStringLiteral("Screenshot");
+    }
+    return truncatedFilename(result);
 }
 
 QString ExportManager::autoIncrementFilename(const QString &baseName, const QString &extension,
                                              FileNameAlreadyUsedCheck isFileNameUsed)
 {
-    if (!((this->*isFileNameUsed)(QUrl::fromUserInput(baseName + '.' + extension)))) {
-        return baseName + '.' + extension;
+    QString result = truncatedFilename(baseName) + QLatin1Literal(".") + extension;
+    if (!((this->*isFileNameUsed)(QUrl::fromUserInput(result)))) {
+        return result;
     }
 
-    QString fileNameFmt(baseName + "-%1." + extension);
+    QString fileNameFmt = truncatedFilename(baseName) + QStringLiteral("-%1.");
     for (quint64 i = 1; i < std::numeric_limits<quint64>::max(); i++) {
-        if (!((this->*isFileNameUsed)(QUrl::fromUserInput(fileNameFmt.arg(i))))) {
-            return fileNameFmt.arg(i);
+        result = fileNameFmt.arg(i) + extension;
+        if (!((this->*isFileNameUsed)(QUrl::fromUserInput(result)))) {
+            return result;
         }
     }
 
     // unlikely this will ever happen, but just in case we've run
     // out of numbers
 
-    return fileNameFmt.arg("OVERFLOW-" + QString::number(qrand() % 10000));
+    result = fileNameFmt.arg(QStringLiteral("OVERFLOW-") + QString::number(qrand() % 10000));
+    return truncatedFilename(result) + extension;
 }
 
 QString ExportManager::makeSaveMimetype(const QUrl &url)
@@ -195,7 +260,7 @@ bool ExportManager::writeImage(QIODevice *device, const QByteArray &format)
 {
     QImageWriter imageWriter(device, format);
     if (!(imageWriter.canWrite())) {
-        emit errorMessage(i18n("QImageWriter cannot write image: ") + imageWriter.errorString());
+        emit errorMessage(i18n("QImageWriter cannot write image: %1", imageWriter.errorString()));
         return false;
     }
 
@@ -247,7 +312,7 @@ QUrl ExportManager::tempSave(const QString &mimetype)
     }
 
     if (!mTempDir) {
-        mTempDir = new QTemporaryDir(QDir::tempPath() + QDir::separator() + "Spectacle.XXXXXX");
+        mTempDir = new QTemporaryDir(QDir::tempPath() + QDir::separator() + QStringLiteral("Spectacle.XXXXXX"));
     }
     if (mTempDir && mTempDir->isValid()) {
         // create the temporary file itself with normal file name and also unique one for this session
@@ -326,10 +391,9 @@ void ExportManager::doSave(const QUrl &url, bool notify)
     }
 }
 
-void ExportManager::doSaveAs(QWidget *parentWindow)
+bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
 {
     QStringList supportedFilters;
-    QMimeDatabase db;
     SpectacleConfig *config = SpectacleConfig::instance();
 
     // construct the supported mimetype list
@@ -354,9 +418,15 @@ void ExportManager::doSaveAs(QWidget *parentWindow)
             if (save(saveUrl)) {
                 emit imageSaved(saveUrl);
                 config->setLastSaveAsLocation(saveUrl.adjusted(QUrl::RemoveFilename));
+
+                if (notify) {
+                    emit forceNotify(saveUrl);
+                }
+                return true;
             }
         }
     }
+    return false;
 }
 
 // misc helpers
