@@ -19,29 +19,24 @@
 
 #include "ExportMenu.h"
 
-#include <QTimer>
-#include <QList>
-#include <QUrl>
-#include <QJsonObject>
-#include <QJsonArray>
-#include <QDebug>
+#include "Config.h"
 
 #include <KLocalizedString>
-#include <KStandardShortcut>
-#include <KService>
 #include <KMimeTypeTrader>
 #include <KRun>
-
-#include "Config.h"
+#include <KStandardShortcut>
 #ifdef KIPI_FOUND
 #include <KIPI/Plugin>
-#include <KIPI/PluginLoader>
-#include "KipiInterface/KSGKipiInterface.h"
 #endif
 
+#include <QDebug>
+#include <QJsonArray>
+#include <QTimer>
+
 ExportMenu::ExportMenu(QWidget *parent) :
     QMenu(parent),
 #ifdef PURPOSE_FOUND
+    mUpdatedImageAvailable(false),
     mPurposeMenu(new Purpose::Menu(this)),
 #endif
     mExportManager(ExportManager::instance())
@@ -68,17 +63,12 @@ void ExportMenu::populateMenu()
 }
 
 
-void ExportMenu::imageUpdated(const QString &dataUri)
+void ExportMenu::imageUpdated()
 {
 #ifdef PURPOSE_FOUND
-    mPurposeMenu->model()->setInputData(QJsonObject {
-        { QStringLiteral("mimeType"), QStringLiteral("image/png") },
-        { QStringLiteral("urls"), QJsonArray({ dataUri }) }
-    });
-    mPurposeMenu->model()->setPluginType("Export");
-    mPurposeMenu->reload();
-#else
-    Q_UNUSED(dataUri);
+    // mark cached image as stale
+    mUpdatedImageAvailable = true;
+    mPurposeMenu->clear();
 #endif
 }
 
@@ -91,8 +81,8 @@ void ExportMenu::getKServiceItems()
     const KService::List services = KMimeTypeTrader::self()->query(QStringLiteral("image/png"));
 
     Q_FOREACH (auto service, services) {
-        QString name = service->name().replace('&', QLatin1String("&&"));
-        QAction *action = new QAction(QIcon::fromTheme(service->icon()), name, nullptr);
+        QString name = service->name().replace(QLatin1Char('&'), QLatin1String("&&"));
+        QAction *action = new QAction(QIcon::fromTheme(service->icon()), name, this);
 
         connect(action, &QAction::triggered, [=]() {
             QList<QUrl> whereIs({ mExportManager->tempSave() });
@@ -108,7 +98,7 @@ void ExportMenu::getKServiceItems()
 
     QAction *openWith = new QAction(this);
     openWith->setText(i18n("Other Application"));
-    openWith->setIcon(QIcon::fromTheme(QStringLiteral("application-x-executable")));
+    openWith->setIcon(QIcon::fromTheme(QStringLiteral("document-share")));
     openWith->setShortcuts(KStandardShortcut::open());
 
     connect(openWith, &QAction::triggered, [=]() {
@@ -123,6 +113,7 @@ void ExportMenu::loadKipiItems()
 {
     if (!mKipiMenuLoaded) {
         QTimer::singleShot(500, this, &ExportMenu::getKipiItems);
+        mKipiMenuLoaded = true;
     }
 }
 
@@ -158,7 +149,7 @@ void ExportMenu::getKipiItems()
             KIPI::Category category = plugin->category(action);
             if (category == KIPI::ExportPlugin) {
                 exportActions += action;
-            } else if (category == KIPI::ImagesPlugin && pluginInfo->library().contains("kipiplugin_sendimages")) {
+            } else if (category == KIPI::ImagesPlugin && pluginInfo->library().contains(QStringLiteral("kipiplugin_sendimages"))) {
                 exportActions += action;
             }
         }
@@ -168,7 +159,10 @@ void ExportMenu::getKipiItems()
         }
     }
 
-    mKipiMenuLoaded = true;
+    // If there are no export actions, then perhaps the kipi-plugins package is not installed.
+    if (mKipiMenu->isEmpty()) {
+        mKipiMenu->addAction(i18n("No KIPI plugins available"))->setEnabled(false);
+    }
 }
 #endif
 
@@ -184,8 +178,29 @@ void ExportMenu::loadPurposeMenu()
         if (error) {
             emit imageShared(true, message);
         } else {
-            emit imageShared(false, output["url"].toString());
+            emit imageShared(false, output[QStringLiteral("url")].toString());
         }
     });
+
+    // update available options based on the latest picture
+    connect(mPurposeMenu, &QMenu::aboutToShow, this, &ExportMenu::loadPurposeItems);
+}
+
+void ExportMenu::loadPurposeItems()
+{
+    if (!mUpdatedImageAvailable) {
+        return;
+    }
+
+    // updated image available, we lazily load it now
+    QString dataUri = ExportManager::instance()->pixmapDataUri();
+    mUpdatedImageAvailable = false;
+
+    mPurposeMenu->model()->setInputData(QJsonObject {
+        { QStringLiteral("mimeType"), QStringLiteral("image/png") },
+        { QStringLiteral("urls"), QJsonArray({ dataUri }) }
+    });
+    mPurposeMenu->model()->setPluginType(QStringLiteral("Export"));
+    mPurposeMenu->reload();
 }
 #endif
