@@ -42,9 +42,11 @@ class KSWidget : public QWidget
     Q_OBJECT
 
 	public:
-    
+
     explicit KSWidget(QWidget *parent = 0);
 
+    int imagePaddingWidth() const;
+
 	signals:
 
     void dragInitiated();
@@ -62,7 +64,7 @@ class KSWidget : public QWidget
     void captureModeChanged(int index);
 
 	private:
-    
+
     QGridLayout   *mMainLayout;
     QHBoxLayout   *mDelayLayout;
     QVBoxLayout   *mRightLayout;
@@ -76,6 +78,7 @@ class KSWidget : public QWidget
     QCheckBox     *mMousePointer;
     QCheckBox     *mWindowDecorations;
     QCheckBox     *mCaptureTransientOnly;
+    QCheckBox     *mQuitAfterSaveOrCopy;
     QLabel        *mCaptureModeLabel;
     QLabel        *mContentOptionsLabel;
 };
